/**
 * LICENCIA LGPL:
 * 
 * Esta librera es Software Libre; Usted puede redistribuirla y/o modificarla
 * bajo los trminos de la GNU Lesser General Public License (LGPL) tal y como 
 * ha sido publicada por la Free Software Foundation; o bien la versin 2.1 de 
 * la Licencia, o (a su eleccin) cualquier versin posterior.
 * 
 * Esta librera se distribuye con la esperanza de que sea til, pero SIN 
 * NINGUNA GARANTA; tampoco las implcitas garantas de MERCANTILIDAD o 
 * ADECUACIN A UN PROPSITO PARTICULAR. Consulte la GNU Lesser General Public 
 * License (LGPL) para ms detalles
 * 
 * Usted debe recibir una copia de la GNU Lesser General Public License (LGPL) 
 * junto con esta librera; si no es as, escriba a la Free Software Foundation 
 * Inc. 51 Franklin Street, 5 Piso, Boston, MA 02110-1301, USA o consulte
 * <http://www.gnu.org/licenses/>.
 *
 * Copyright 2011 Agencia de Tecnologa y Certificacin Electrnica
 */
package es.accv.arangi.certificate;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.InputStream;
import java.security.cert.X509Certificate;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;

import org.apache.log4j.Logger;
import org.bouncycastle.asn1.ASN1ObjectIdentifier;
import org.bouncycastle.asn1.x509.X509Name;

import es.accv.arangi.base.certificate.Certificate;
import es.accv.arangi.base.certificate.validation.CAList;
import es.accv.arangi.base.exception.certificate.CertificateCANotFoundException;
import es.accv.arangi.base.exception.certificate.CertificateFieldException;
import es.accv.arangi.base.exception.certificate.NormalizeCertificateException;
import es.accv.arangi.base.exception.certificate.ValidationXMLException;
import es.accv.arangi.base.util.AlternativeNameElement;
import es.accv.arangi.certificate.field.DatosRepresentacionBoletinOficial;
import es.accv.arangi.certificate.field.DatosRepresentacionDesconocido;
import es.accv.arangi.certificate.field.DatosRepresentacionRegistro;
import es.accv.arangi.certificate.field.DatosRepresentacionRegistroNotarial;
import es.accv.arangi.exception.ResourceNotLoadedException;
import es.accv.arangi.util.ArangiUtil;

/**
 * Clase para el tratamiento de los certificados reconocidos de representante de la ACCV, segn la 
 * poltica definida en la URL: 
 * <a href="http://www.accv.es/quienes-somos/practicas-y-politicas-de-certificacion/politicas-de-certificacion/" target="politica">
 * http://www.accv.es/quienes-somos/practicas-y-politicas-de-certificacion/politicas-de-certificacion/</a>
 * 
 * @author <a href="mailto:jgutierrez@accv.es">Jos M Gutirrez</a>
 */
public class CertificadoRepresentante extends CertificadoPersona{

	/*
	 * Logger de la clas
	 */
	static Logger logger = Logger.getLogger(CertificadoRepresentante.class);
	
	/**
	 * Alias del keystore donde se guarda el certificado
	 */
	public static final String ALIAS_PKCS11 = "REPR";

	/**
	 * Base del OID de la poltica de certificados de ep en dispositivo seguro
	 */
	public static final String POLICY_WITH_IN_PKCS11_DEVICE = "1.3.6.1.4.1.8149.3.29";

	/**
	 * Base del OID de la poltica de certificados de ep en dispositivo software
	 */
	public static final String POLICY_WITH_IN_SOFTWARE_DEVICE = "1.3.6.1.4.1.8149.3.30";

	/**
	 * Base del OID de la poltica de certificados de ep en dispositivo seguro
	 */
	public static final String POLICY_WITHOUT_IN_PKCS11_DEVICE = "1.3.6.1.4.1.8149.3.31";

	/**
	 * Base del OID de la poltica de certificados de ep en dispositivo software
	 */
	public static final String POLICY_WITHOUT_IN_SOFTWARE_DEVICE = "1.3.6.1.4.1.8149.3.32";

	/*
	 * Lista de SKI de los certificados de CA de explotacin
	 */
	private static HashSet setExplotationCACertificates = new HashSet();
	
	/*
	 * Lista de SKI de los certificados de CA de test
	 */
	private static HashSet setTestCACertificates = new HashSet();
	
	/**
	 * Constructor con un certificado X509Certificate
	 * 
	 * @param certificate Certificado en formato X.509
	 * @throws CertificateCANotFoundException No se ha encontrado alguno de los certificados de
	 * las CA que tratan este certificado en el classpath
	 * @throws NormalizeCertificateException El certificado no puede ser normalizado al formato
	 * 	reconocido por el proveedor criptogrfico de Arangi o su firma no es correcta o no
	 * 	puede ser analizada
	 */
	public CertificadoRepresentante(X509Certificate certificate) throws CertificateCANotFoundException, NormalizeCertificateException {
		super(certificate, getCAList());
	}
	
	/**
	 * Constructor con un fichero que contiene un certificado
	 * 
	 * @param fileCertificate Fichero que contiene un certificado en formato X.509
	 * @throws CertificateCANotFoundException No se ha encontrado alguno de los certificados de
	 * las CA que tratan este certificado en el classpath
	 * @throws NormalizeCertificateException El certificado no puede ser normalizado al formato
	 * 	reconocido por el proveedor criptogrfico de Arangi o su firma no es correcta o no
	 * 	puede ser analizada
	 * @throws FileNotFoundException El fichero no existe
	 */
	public CertificadoRepresentante(File fileCertificate) throws CertificateCANotFoundException, NormalizeCertificateException, FileNotFoundException {
		super(fileCertificate, getCAList());
	}
	
	/**
	 * Constructor con un fichero que contiene un certificado
	 * 
	 * @param isCertificate Stream de lectura a un certificado en formato X.509
	 * @throws CertificateCANotFoundException No se ha encontrado alguno de los certificados de
	 * las CA que tratan este certificado en el classpath
	 * @throws NormalizeCertificateException El certificado no puede ser normalizado al formato
	 * 	reconocido por el proveedor criptogrfico de Arangi o su firma no es correcta o no
	 * 	puede ser analizada
	 */
	public CertificadoRepresentante(InputStream isCertificate) throws CertificateCANotFoundException, NormalizeCertificateException {
		super(isCertificate, getCAList());
	}
	
	/**
	 * Constructor con un fichero que contiene un certificado
	 * 
	 * @param contenidoCertificado Contenido de un certificado en formato X.509
	 * @throws CertificateCANotFoundException No se ha encontrado alguno de los certificados de
	 * las CA que tratan este certificado en el classpath
	 * @throws NormalizeCertificateException El certificado no puede ser normalizado al formato
	 * 	reconocido por el proveedor criptogrfico de Arangi o su firma no es correcta o no
	 * 	puede ser analizada
	 */
	public CertificadoRepresentante(byte[] contenidoCertificado) throws CertificateCANotFoundException, NormalizeCertificateException {
		super(contenidoCertificado, getCAList());
	}
	
	//-- Mtodos pblicos
	
	/**
	 * Comprueba si el certificado es de una CA de test
	 * 
	 * @return Cierto si el certificado es de una CA de test
	 */
	public boolean isTestCertificate () {
		logger.debug("[CertificadoCiudadano.isTestCertificate]::Entrada");
		return setTestCACertificates.contains(getIssuerKeyIdentifier());
	}
	
	/**
	 * Mtodo que devuelve el nombre del titular del certificado
	 * 
	 * @return Nombre del titular del certificado
	 */
	public String getName () {
		return getElementSubject(X509Name.GIVENNAME);
	}
	
	/**
	 * Mtodo que devuelve el primer apellido del titular del certificado
	 * 
	 * @return Primer apellido del titular del certificado
	 */
	public String getFirstSurname () {
		
		logger.debug ("[CertificadoRepresentante.getFirstSurname]::Entrada");
		
		return getNombreCompleto()[1].trim();
	}
	
	/**
	 * Mtodo que devuelve el segundo apellido del titular del certificado
	 * 
	 * @return Segundo apellido del titular del certificado
	 */
	public String getSecondSurname () {
		
		logger.debug ("[CertificadoRepresentante.getSecondSurname]::Entrada");
		
		String[] nombreCompleto = getNombreCompleto();
		
		//-- El segundo apellido no es obligatorio
		if (nombreCompleto == null || nombreCompleto.length < 3) {
			return "";
		}
		
		return nombreCompleto[2].trim();
	}
	
	/**
	 * Mtodo que devuelve los apellidos del representante de la entidad
	 * 
	 * @return Apellidos del representante de la entidad
	 */
	public String getSurnames () {
		
		logger.debug ("[CertificadoRepresentante.getSurnames]::Entrada");
		
		return getFirstSurname() + " " + getSecondSurname();
	}
	
	/**
	 * Mtodo que devuelve el NIF del representante de la entidad
	 * 
	 * @return NIF del titular del certificado
	 */
	public String getNIF () {
		
		logger.debug ("[CertificadoRepresentante.getNIF]::Entrada");

		return getElementSubject(X509Name.SERIALNUMBER);
	}
	
	/**
	 * Mtodo que obtiene el CIF de la entidad.
	 * 
	 * @return CIF de la entidad
	 */
	public String getEntityCIF() {
		
		logger.debug ("[CertificadoRepresentante.getCIF]::Entrada");
		
		return getElementSubject(new ASN1ObjectIdentifier("2.5.4.97")).replace("VATES-", "");
	}
	
	/**
	 * Mtodo que obtiene el CIF de la entidad codificado segn el ETSI
	 * (slo para certificados cualificados de sello de entidad).
	 * 
	 * @return CIF de la entidad segn el ETSI
	 */
	public String getEntityCIFETSI() {
		
		logger.debug ("[CertificadoRepresentante.getEntityCIFETSI]::Entrada");
		
		return getElementSubject(new ASN1ObjectIdentifier("2.5.4.97"));
	}
	
	/**
	 * Mtodo que obtiene el nombre de la entidad.
	 * 
	 * @return Nombre de la entidad
	 */
	public String getEntityName() {
		
		logger.debug ("[CertificadoRepresentante.getEntityName]::Entrada");
		
		return getElementSubject(X509Name.O);
	}
	
	/**
	 * Obtiene la informacin que relaciona al representante con la entidad en
	 * los certificados cualificados de entidad como una tira de texto.
	 * 
	 * @return Tira de texto con la informacin de representacin
	 */
	public String getRepresentationDataString() {
		return getElementSubject(new ASN1ObjectIdentifier("2.5.4.13"));
	}
	
	/**
	 * Obtiene la informacin que relaciona al representante con la entidad en
	 * los certificados cualificados de entidad. Estos datos se incluyen en el 
	 * certificado y pueden ser una entrada en un registro, un registro notarial, 
	 * una entrada en un boletn o cualquier otra cosa.<br>
	 * Los certificados generados antes de Julio de 2016 no dispondrn de este
	 * campo.
	 * 
	 * @return En caso de que exista el campo se devolver un objeto que puede ser
	 * 	de las siguientes clases: DatosRepresentacionRegistro, DatosRepresentacionRegistroNotarial,
	 * 	DatosRepresentacionBoletinOficial o DatosRepresentacionDesconocido. Si el
	 * 	campo no existe se devolver null.
	 */
	public Object getRepresentationData() {
		String descripcion = getRepresentationDataString();
		logger.debug ("[CertificadoEntidad.getRepresentationData]::Descripcion: " + descripcion);
		if (descripcion == null) {
			return null;
		}
		
		String[] elementos = descripcion.split("/");
		if (descripcion.startsWith("R:")) {
			DatosRepresentacionRegistro datos = new DatosRepresentacionRegistro();
			datos.setRegistro(elementos[0].substring(elementos[0].indexOf(":") + 1));
			datos.setHoja(elementos[1]);
			datos.setTomo(elementos[2]);
			datos.setSeccion(elementos[3]);
			datos.setLibro(elementos[4]);
			datos.setFolio(elementos[5]);
			if (!elementos[6].trim().equals("")) {
				try {
					datos.setFecha(ArangiUtil.SIMPLE_DATE_FORMAT.parse(elementos[6]));
				} catch (ParseException e) {
					logger.info("No se puede parsear la fecha '" + elementos[6] + "");
				}
			}
			datos.setInscripcion(elementos[7]);
			return datos;
		} else if (descripcion.startsWith("N:")) {
			DatosRepresentacionRegistroNotarial datos = new DatosRepresentacionRegistroNotarial();
			datos.setNotario(elementos[0].substring(elementos[0].indexOf(":") + 1));
			datos.setNumeroProtocolo(elementos[1]);
			if (!elementos[2].trim().equals("")) {
				try {
					datos.setFechaOtorgamiento(ArangiUtil.SIMPLE_DATE_FORMAT.parse(elementos[2]));
				} catch (ParseException e) {
					logger.info("No se puede parsear la fecha '" + elementos[6] + "");
				}
			}
			return datos;
		} else if (descripcion.startsWith("B:")) {
			DatosRepresentacionBoletinOficial datos = new DatosRepresentacionBoletinOficial();
			datos.setBoletin(elementos[0].substring(elementos[0].indexOf(":") + 1));
			if (!elementos[1].trim().equals("")) {
				try {
					datos.setFecha(ArangiUtil.SIMPLE_DATE_FORMAT.parse(elementos[1]));
				} catch (ParseException e) {
					logger.info("No se puede parsear la fecha '" + elementos[6] + "");
				}
			}
			datos.setNumeroResolucion(elementos[2]);
			return datos;
		} else {
			DatosRepresentacionDesconocido datos = new DatosRepresentacionDesconocido(descripcion);
			return datos;
		}
	}
	
	/**
	 * Mtodo que devuelve la direccin de correo electrnico de la entidad
	 * 
	 * @return E-mail de la entidad titular del certificado
	 */
	public String getEmail () {
		
		logger.debug ("[CertificadoRepresentante.getEmail]::Entrada");
		
		List altNames;
		try {
			altNames = getSubjectAlternativeName();
		} catch (CertificateFieldException e) {
			logger.info ("[CertificadoRepresentante.getEmail]::No ha sido posible obtener el e-mail de la entidad", e);
			return null;
		}
	
		//-- Es el primer elemento del nombre alternativo
		return (String) ((AlternativeNameElement) altNames.get(0)).getValue();
	}
	
	/**
	 * El certificado se encuentra en un dispositivo PKCS#11
	 * 
	 * @return Cierto si el certificado se encuentra en un dispositivo PKCS#11
	 */
	public boolean isInPkcs11Device () {
		logger.debug ("[CertificadoRepresentante.isInPkcs11Device]::Entrada");
		
		for (String policyOID : getPolicyOIDs()) {
			if (policyOID.startsWith(POLICY_WITH_IN_PKCS11_DEVICE) ||
					policyOID.startsWith(POLICY_WITHOUT_IN_PKCS11_DEVICE)) {
				return true;
			}
		}
		
		return false;
	}
	
	/**
	 * El certificado se encuentra en un dispositivo software (PKCS#12)
	 * 
	 * @return Cierto si el certificado se encuentra en un dispositivo software (PKCS#12)
	 */
	public boolean isInSoftwareDevice () {
		logger.debug ("[CertificadoRepresentante.isInSoftwareDevice]::Entrada");
		
		for (String policyOID : getPolicyOIDs()) {
			if (policyOID.startsWith(POLICY_WITH_IN_SOFTWARE_DEVICE) ||
					policyOID.startsWith(POLICY_WITHOUT_IN_SOFTWARE_DEVICE)) {
				return true;
			}
		}
		
		return false;
	}
	
	/**
	 * El certificado se emiti para una entidad con personalidad jurdica
	 * 
	 * @return Cierto si el certificado se emiti para una entidad con personalidad jurdica
	 */
	public boolean isWithLegalPersonality () {
		logger.debug ("[CertificadoRepresentante.isWithLegalPersonality]::Entrada");
		
		for (String policyOID : getPolicyOIDs()) {
			if (policyOID.startsWith(POLICY_WITH_IN_PKCS11_DEVICE) ||
					policyOID.startsWith(POLICY_WITH_IN_SOFTWARE_DEVICE)) {
				return true;
			}
		}
		
		return false;
	}
	
	/**
	 * El certificado se emiti para una entidad sin personalidad jurdica
	 * 
	 * @return Cierto si el certificado se emiti para una entidad sin personalidad jurdica
	 */
	public boolean isWithoutLegalPersonality () {
		logger.debug ("[CertificadoRepresentante.isWithoutLegalPersonality]::Entrada");
		
		for (String policyOID : getPolicyOIDs()) {
			if (policyOID.startsWith(POLICY_WITHOUT_IN_PKCS11_DEVICE) ||
					policyOID.startsWith(POLICY_WITHOUT_IN_SOFTWARE_DEVICE)) {
				return true;
			}
		}
		
		return false;
	}
	
	
	@Override
	public boolean isCipherCertificate() {
		return false;
	}

	@Override
	public boolean isSigningCertificate() {
		return true;
	}
	
	/**
	 * Obtiene la lista de certificados de CA y raz que conforman las posibles cadenas
	 * de confianza de los certificados de esta clase. 
	 */
	public static CAList getCAList() throws CertificateCANotFoundException {
		
		//-- Aadir los certificados de test y de explotacin
		List lCACertificates = getCAListExplotation();
		lCACertificates.addAll(getCAListTest());
		
		CAList caList;
		try {
			caList = new CAList (lCACertificates);
		} catch (NormalizeCertificateException e) {
			//-- Si algn certificado no puede ser normalizado lo pasamos como que no
			//-- se ha podido cargar
			logger.info ("[CertificadoRepresentante.getCAList]::Alguno de los certificados de las CA no ha podido ser " +
					"normalizado a lo esperado por el proveedor criptogrfico de Arangi", e);
			throw new CertificateCANotFoundException ("Alguno de los certificados de las CA no ha podido ser " +
					"normalizado a lo esperado por el proveedor criptogrfico de Arangi", e);
		};
		
		//-- Aadir el fichero para validar los certificados de test
		try {
			caList.setValidationXML(ArangiUtil.loadFile("file/validation_data_accv_test.xml"));
		} catch (ValidationXMLException e) {
			//-- Si no se ha podido parsear el fichero de validacin lo pasamos como que
			//-- no se han podido cargar los certificados de las CA
			logger.info ("[CertificadoRepresentante.getCAList]::No ha sido posible parsear el fichero de validacin XML", e);
			throw new CertificateCANotFoundException ("No ha sido posible parsear el fichero de validacin XML", e);
		} catch (ResourceNotLoadedException e) {
			//-- Si no se encuentra pasaremos sin l, slo que no funcionarn los certificados de test
			logger.info ("[CertificadoRepresentante.getCAList]::No ha sido posible obtener el fichero de validacin XML", e);
		}
		
		return caList;
	}

	//-- Mtodos protected
	
	/**
	 * Usado por la clase CertificateFactory para dar de alta la clase en la lista
	 * de tipos de certificados.
	 * 
	 * @return OID base de la poltica
	 */
	protected static String [] getBasePolicies () {
		return new String[] { 
				POLICY_WITH_IN_PKCS11_DEVICE, // certificados en tarjeta
				POLICY_WITH_IN_SOFTWARE_DEVICE, // certificados en software
				POLICY_WITHOUT_IN_PKCS11_DEVICE, // certificados en tarjeta
				POLICY_WITHOUT_IN_SOFTWARE_DEVICE // certificados en software
		};
	}

	//-- Mtodos privados
	
	/*
	 * Obtiene la lista de certificados de CA y raz que conforman las posibles cadenas
	 * de confianza de los certificados de explotacin de esta clase. 
	 */
	private static List getCAListExplotation() throws CertificateCANotFoundException {
		
		//-- Aadir los certificados de explotacin
		List lCACertificates = new ArrayList ();
		X509Certificate certificate = ArangiUtil.loadCertificate("certificate/ACCV-CA120-SHA256");
		setExplotationCACertificates.add(Certificate.getSubjectKeyIdentifier(certificate));
		lCACertificates.add(certificate);
		
		certificate = ArangiUtil.loadCertificate("certificate/ACCVRAIZ1");
		setExplotationCACertificates.add(Certificate.getSubjectKeyIdentifier(certificate));
		lCACertificates.add(certificate);
		
		return lCACertificates;
	}
	
	/*
	 * Obtiene la lista de certificados de CA y raz que conforman las posibles cadenas
	 * de confianza de los certificados de test de esta clase. 
	 */
	private static List getCAListTest() throws CertificateCANotFoundException {
		
		//-- Aadir los certificados de test y de explotacin
		List lCACertificates = new ArrayList ();
		X509Certificate certificate = ArangiUtil.loadCertificate("certificate/ACCVCATEST120");
		setTestCACertificates.add(Certificate.getSubjectKeyIdentifier(certificate));
		lCACertificates.add(certificate);
		
		certificate = ArangiUtil.loadCertificate("certificate/ROOTEJB4TEST");
		setTestCACertificates.add(Certificate.getSubjectKeyIdentifier(certificate));
		lCACertificates.add(certificate);
		
		return lCACertificates;
	}

	/*
	 * Obtiene el nombre completo que hay en la extensin Subject Alternative Name como
	 * entrada del LDAP. El array tendr 3 elementos: nombre, apellido 1 y apellido 2
	 */
	private String [] getNombreCompleto () {
		
		String nombreCompleto;
		try {
			nombreCompleto = getSubjectAlternativeNameElement(OID_ID_AT_COMMONNAME);
		} catch (CertificateFieldException e) {
			logger.info ("[CertificadoRepresentante.getNombreCompleto]::No ha sido posible obtener el nombre completo del ciudadano");
			return null;
		}
		if (nombreCompleto == null || nombreCompleto.length() == 0) {
			logger.info ("[CertificadoRepresentante.getNombreCompleto]::No ha sido posible obtener el nombre completo del ciudadano");
			return null;
		}
		return nombreCompleto.split("\\|");

	}

}
